@@ -2,7 +2,8 @@
 GeoUtils is a python package of raster and vector tools.
 """
 
-from geoutils import datasets, georaster, geovector, satimg  # noqa
+from geoutils import spatial_tools  # noqa
+from geoutils import datasets, georaster, geovector, projtools, satimg  # noqa
 from geoutils.georaster import Raster  # noqa
 from geoutils.geovector import Vector  # noqa
 from geoutils.satimg import SatelliteImage  # noqa
